import sys

if sys.platform == "win32":
    from rerain.functions.sys.audio import mute, unmute
    from rerain.functions.sys.mic import micmute, micunmute, record
else:
    mute = None
    unmute = None

from rerain.functions.math.math import add
from rerain.functions.math.math import sub
from rerain.functions.math.math import mult
from rerain.functions.math.math import div
from rerain.functions.math.math import power
from rerain.functions.math.math import sqrt
from rerain.functions.math.math import abs_value
from rerain.functions.math.math import log
from rerain.functions.math.math import log10
from rerain.functions.math.math import sin
from rerain.functions.math.math import cos
from rerain.functions.math.math import tan
from rerain.functions.math.math import atan
from rerain.functions.math.math import nth_root
from rerain.functions.math.math import factorial
from rerain.functions.math.math import exp
from rerain.functions.math.math import pi
from rerain.functions.math.math import e
from rerain.functions.math.math import sin_deg
from rerain.functions.math.math import cos_deg
from rerain.functions.math.math import tan_deg
from rerain.functions.math.math import derivative
from rerain.functions.math.math import integral
from rerain.functions.math.math import gcd
from rerain.functions.math.math import lcm
from rerain.functions.math.math import combination
from rerain.functions.math.math import permutation
from rerain.functions.qr.qr import newqr
from rerain.functions.sys.sys import find
from rerain.functions.sys.sys import findf
from rerain.functions.sys.sys import findall
from rerain.functions.sys.sys import create
from rerain.functions.sys.sys import rm
from rerain.functions.sys.time import wait
from rerain.functions.sys.time import wait_seconds
from rerain.functions.sys.time import wait_minutes
from rerain.functions.sys.time import wait_hours
from rerain.functions.sys.time import ct

__all__ = [
    "add", "sub", "mult", "div", "power", "sqrt", "abs_value", "log", "log10",
    "sin", "cos", "tan", "atan", "nth_root", "factorial", "exp", "pi", "e",
    "sin_deg", "cos_deg", "tan_deg", "derivative", "integral", "gcd", "lcm",
    "combination", "permutation", "newqr", "find", "findf", "findall", "create",
    "rm", "wait", "wait_seconds", "wait_minutes", "wait_hours", "ct", "micmute", "micunmute", "record"
]

if sys.platform == "win32":
    __all__.append("mute")
    __all__.append("unmute")
    __all__.append("micmute")
    __all__.append("micunmute")
    __all__.append("record")